(* Mathematica package *)

BeginPackage["AtomicDensityMatrix`PolarizationMoments`",{
  (*"PhysicalConstants`","Units`",*)
  "AtomicDensityMatrix`",
  "AtomicDensityMatrix`Common`",
  "AtomicDensityMatrix`CouplingCoefficients`",
  "AtomicDensityMatrix`TensorAlgebra`",
  "AtomicDensityMatrix`AtomicSystem`",
  "AtomicDensityMatrix`ExperimentalSystem`",
  "AtomicDensityMatrix`DensityMatrix`",
  "AtomicDensityMatrix`WignerEckart`", 
  "AtomicDensityMatrix`WignerRotate`"}]

Begin["`Private`"]

(* ::Subsection:: *)
(*Tensor decomposition*)


Options[PolarizationMoments]=DeleteCases[Options[DensityMatrix]/.(r_->_)->(r->Automatic),Representation->_]

PolarizationMoments[j_?HalfIntegerQ,opts___Rule] := PolarizationMoments[ZeemanSystem[j], opts]

PolarizationMoments[syst:{_AtomicState..},opts___Rule] :=
	Module[{newopts, rfunc},
		newopts = 
			Join[
	   			DeleteCases[Join[{opts},Options[PolarizationMoments]],_->Automatic],
	   			Options[DensityMatrix]
	   		]/.(DMLabel->(None|False))->(DMLabel->Sequence[]);
	    rfunc[l1_,l2_,k_,q_] = 
	    	(Subscript[DMSymbol, DMLabel,{l1,l2},{k,q}][TimeVariable]/.newopts)[[If[TimeDependence/.newopts, All, 0]]];
	    DirectSelfProduct[
	    	Table[
	      		Transpose@{Table[rfunc[#1[[1]],#2[[1]],k,q],{q,k,-k,-1}]},
	      		{k,Abs[#1[[2]]-#2[[2]]],#1[[2]]+#2[[2]]}
	      	]&,
	      	{LevelLabel[#],F[#]}&/@RemoveZeemanSublevels[syst]
	    ]
	]

PolarizationMatrices[j_?HalfIntegerQ] := Component@PolarizationMatrices[ZeemanSystem[j]]

PolarizationMatrices[syst:{_AtomicState..}] :=
   Module[{r},
      ECT[PolarizationMoments[syst, TimeDependence -> False, DMSymbol->r]] /. 
      	Subscript[r, {s1_, s2_}, {k_, q_}] :> WignerEckart[syst, {Polarization[s1, s2], k, q}]
   ]

(*Convert between m m' and \[Kappa] q representation (contravariant components) using the polarization operator. [Var88, Sec. 6.1.5, Eqs. (49,50)], also [Ale2004, Eqs. (A3-8) ]*)

Decomposition[syst:{_AtomicState..},X_?OperatorQ]:=
	Map[Tr[#\[CenterDot]X]&,ECT@PolarizationMatrices[syst],{5}]

Decomposition[N_?OperatorQ]:=Component@Decomposition[ZeemanSystem[Rank[N]],N]

Recomposition[X_?TensorDecompositionQ]:=Recomposition[ZeemanSystem[Rank[X]],{{X}}]

Recomposition[syst:{_AtomicState..},X_?ContravariantDecompositionMatrixQ]:=PolarizationMatrices[syst]\[CenterDot]X

Recomposition[syst:{_AtomicState..},X_?CovariantDecompositionMatrixQ]:=X\[CenterDot]ECT@PolarizationMatrices[syst]

Recomposition[J_?HalfIntegerQ,X_?ContravariantQ]:=Component[PolarizationMatrices[J],Rank[X]]\[CenterDot]X

Recomposition[J_?HalfIntegerQ,X_?CovariantQ]:=X\[CenterDot]ECT@Component[PolarizationMatrices[J],Rank[X]]


(* ::Subsection:: *)
(*Angular momentum probability distribution*)


(*Angular momentum probability distribution in terms of the \[Kappa]-q tensor elements [Ale2004, Eq. (A11)]*)

\[Rho]JJ=MaxProjectionProbability;

MaxProjectionProbability[T_?TensorDecompositionQ,th_,ph_]:=Module[
	{J=Rank[T],T1=ToContravariant@T},
	Sqrt[(4 Pi)/(2J+1)]
	*Sum[
		CG[{J,J},{k,0},{J,J}]Component[T1,k,q]SphericalHarmonicY[k,q,th,ph],
		{k,0,2J},{q,-k,k}
	]
]

(*Angular momentum probability distribution in terms of the m m' tensor elements [Ale2004, Eq. (A12)]*)

MaxProjectionProbability[\[Rho]_?OperatorQ,\[Theta]_,ph_]:=
	(ECT[#]\[CenterDot]\[Rho]\[CenterDot]#&)[WignerD[Rank[\[Rho]],{ph,\[Theta],0}]\[CenterDot]StretchedState[Rank[\[Rho]]]]

Options[AMPSPlot]=Options[SphericalPlot3D]

AMPSPlot[\[Rho]_,opts___Rule]:=
	SphericalPlot3D[Evaluate[Re[MaxProjectionProbability[\[Rho],\[Theta],ph]]],{\[Theta],0,\[Pi]},{ph,0,2\[Pi]},opts,Evaluate[Sequence@@Options[AMPSPlot]]]

End[]

EndPackage[]
