(* ::Package:: *)

(* ::Title:: *)
(*Arrow3D*)


(* ::Section::Closed:: *)
(*Reference*)


(* ::Subsection:: *)
(*Title*)


(* ::Text:: *)
(*Axes3D.m*)


(* ::Subsection:: *)
(*Author*)


(* ::Text:: *)
(*Simon Rochester (simonkeys@yahoo.com)*)


(* ::Subsection:: *)
(*Summary*)


(* ::Text:: *)
(*The Axes3D package provides functions for adding axes arrows to 3D plots.*)


(* ::Subsection:: *)
(*Copyright*)


(* ::Text:: *)
(*\[Copyright] Copyright 2004-2005, Simon Rochester*)


(* ::Subsection:: *)
(*Notebook Version*)


(* ::Text:: *)
(*1.0*)


(* ::Subsection:: *)
(*Mathematica version*)


(* ::Text:: *)
(*5.0-5.1*)


(* ::Subsection:: *)
(*History*)


(* ::Text:: *)
(*1.0: First version set up as a package (2005-4-24)*)


(* ::Subsection:: *)
(*Keywords*)


(* ::Text:: *)
(*Graphics3D, axes*)


(* ::Section:: *)
(*Interface*)


(* ::Text:: *)
(*This part declares publicly visible symbols.*)


(* ::Subsection:: *)
(*Begin package context*)


BeginPackage["Graphics`Arrow3D`",{"Graphics`GraphicsUtilities`"}]


(* ::Subsection:: *)
(*Usage messages*)


Axes3D::usage=""


AutoLengthScale::usage=""


RenderStyle::usage=""


Arrow3D::usage=""


ArrowShape::usage=""


Width::usage=""


LabelPosition::usage=""


HeadAngle::usage=""


ColorAll::usage=""


FatHead::usage=""


FlatHead::usage=""


Opening::usage=""


Cone3D::usage=""


Disk3D::usage=""


Circle3D::usage=""


OutlineShadow::usage=""


WireCylinder::usage=""


WireFrame::usage=""


fixArrowheadSpecs::usage=""


TripleOption::usage=""


(* ::Subsection:: *)
(*Error messages*)


(* ::Section:: *)
(*Implementation*)


(* ::Subsection:: *)
(*Begin private context*)


Begin["`Private`"]


RotateShape[g_,al_,be_,ga_]:=Rotate[#,ga,{0,0,-1}]&@Rotate[#,be,{-1,0,0}]&@Rotate[g,al,{0,0,-1}]


TranslateShape[g_,{x_,y_,z_}]:=Translate[g,{x,y,z}]


(* ::Subsection:: *)
(*Cone3D*)


Options[Cone3D]={Opening->Down,PlotPoints->Automatic,RenderStyle->Full}


Cone3D[r_,h_,opts:OptionsPattern[]]:=Module[{rp3dopts=FilterRules[Join[{opts},Options[Cone3D]],Options[RevolutionPlot3D]],pp},
If[OptionValue[RenderStyle]===Full,
If[OptionValue[Opening]===Down,RevolutionPlot3D[h (1-t/r),{t,0,r},Mesh->False,Evaluate[Sequence@@rp3dopts]][[1]],
RevolutionPlot3D[h (t/r),{t,0,r},Mesh->False,Evaluate[Sequence@@rp3dopts]][[1]]
],
pp=OptionValue[PlotPoints]/.Automatic->10;
If[OptionValue[Opening]===Down,
{Circle3D[{0,0,0},r],Table[Line[{{0,0,h},{r Sin[t],r Cos[t],0}}],{t,0,2\[Pi]-2\[Pi]/pp,2\[Pi]/pp}]},
{Circle3D[{0,0,h},r],Table[Line[{{0,0,0},{r Sin[t],r Cos[t],h}}],{t,0,2\[Pi]-2\[Pi]/pp,2\[Pi]/pp}]}
]
]
]


(* ::Subsection:: *)
(*Disk3D*)


Disk3D[{x_,y_,z_},r_]:=Translate[RevolutionPlot3D[0,{t,0,r},Mesh->False][[1]],{x,y,z}]


(* ::Subsection:: *)
(*WireCylinder*)


WireCylinder[{p1:{x1_,y1_,z1_},p2:{x2_,y2_,z2_}},opts___Rule]:=WireCylinder[{p1,p2},1,opts]


WireCylinder[{p1:{x1_,y1_,z1_},p2:{x2_,y2_,z2_}},r_,opts___Rule]:=Translate[Rotate[WireCylinder[Sqrt[Total[(p2-p1)^2]],r,opts],{{0,0,1},(p2-p1)/Sqrt[Total[(p2-p1)^2]]}],p1]


WireCylinder[h_,r_,opts___Rule]:=RevolutionPlot3D[{r,t},{t,0,h},Mesh->False][[1]]/.Polygon->Line


(* ::Subsection:: *)
(*OutlineShadow*)


Options[OutlineShadow]={BaseStyle->Directive[White,AbsoluteThickness[4]]}


OutlineShadow[vp:{_,_,_},d_,OptionsPattern[]][g_]:={g,Dashing[{}],OptionValue[BaseStyle],GeometricTransformation[g/.{GrayLevel[_]->{},RGBColor[__]->{},Thickness->{},AbsoluteThickness->{},Text[__]->{},Polygon->Line,Cylinder->WireCylinder},{(1+d)IdentityMatrix[3],-d vp}]}


(* ::Subsection:: *)
(*Arrow3D*)


Options[Arrow3D]={ArrowShape->{.1,.03,.01},Width->.01,Label->None,
	LabelPosition->{1,10,\[Pi]/2},LabelStyle->{},RenderStyle->Full,
	Arrowheads->Automatic,BaseStyle->{},PlotPoints->5,Scale->Automatic};


Arrow3D[opts___Rule]:=Arrow3D[{{0,0,0},{1,0,0}},opts]


Arrow3D[p:{x_,y_,z_},opts___Rule]:=Arrow3D[{{0,0,0},p},opts]


Arrow3D[{p1:{x1_,y1_,z1_},p2:{x2_,y2_,z2_}},opts___Rule]/;Norm[p1-p2]==0.:={}


Arrow3D[{p1:{x1_,y1_,z1_},p2:{x2_,y2_,z2_}},opts___Rule]:=
	Translate[Rotate[Arrow3D[Sqrt[Total[(p2-p1)^2]],opts],{{0,0,1},(p2-p1)/Sqrt[Total[(p2-p1)^2]]}],p1]


FatHead[length_,width_,inset_,opts___Rule]:=TranslateZ[-inset-length]@{Cone3D[width,inset,Opening->Down,opts],Cone3D[width,inset+length,Opening->Down,opts]}


FlatHead[length_,width_,inset_,opts___Rule]:=TranslateZ[-length]@Polygon[{{-width,0,-inset},{0,0,length},{width,0,-inset},{0,0,0},{-width,0,-inset}}]


fixArrowheadSpec[s_,i_,n_,default_,scale_]:=Module[{spec=s,defs},
	If[Head[spec]=!=List,spec={spec}];
	defs={1,If[n==1,1,(i-1)/(n-1)],default,{0,0}};
	spec=Join[spec,Drop[defs,Length[spec]]];
	spec=MapThread[(#1/.Automatic->#2)&,{spec,defs}];
	{spec[[2]],scale spec[[1]]spec[[3]],spec[[4]]}
]


fixArrowheadSpecs[s_,default_,scale_,length_]:=Module[{heads=s},
	heads=If[Head[heads]=!=List,{heads},heads];
	heads=Sort[
		MapIndexed[fixArrowheadSpec[#1,#2[[1]],Length[heads],default,scale]&,heads]/.
			Automatic->length,
		OrderedQ[{#1[[2]],#2[[2]]}]&
		]
]


Arrow3D[length_,opts:OptionsPattern[]]:=
	Module[{scale,width,labpos,labstyle,heads=OptionValue[Arrowheads],
		rs=OptionValue[RenderStyle],basestyle},
	{width,scale,labpos,labstyle,basestyle}=
			{Scale Width,Scale,LabelPosition,LabelStyle,BaseStyle}/.
				{opts}/.Options[Arrow3D]/.Automatic->length;
	(*Massage arrowheads specifications*)
	heads=fixArrowheadSpecs[OptionValue[Arrowheads],OptionValue[ArrowShape],OptionValue[Scale],length];
	(*Graphics statement*)
	{If[Head[basestyle]===List,Sequence@@basestyle,basestyle],
	(*shorten arrow shaft if there are arrowheads at head or tail*)
	Switch[rs,Full|FlatHead,{EdgeForm[],Cylinder[#,width]},Flat,{Line[#]}]&@{{0,0,If[Length[heads]>0&&heads[[1,1]]==0&&heads[[1,2,1]]<0,-heads[[1,2,1]],0]},{0,0,length-If[Length[heads]>0&&heads[[-1,1]]==1&&heads[[-1,2,1]]>0,heads[[-1,2,1]],0]}},
	TranslateZ[length #[[1]]]@RotateZ[#[[3,2]]]@RotateX[#[[3,1]]]@Switch[rs,Full,FatHead,Flat|FlatHead,FlatHead][Sequence@@#[[2]],PlotPoints->OptionValue[PlotPoints]]&/@heads,
	If[OptionValue[Label]=!=None,RotateZ[labpos[[3]]]@Style[Text[OptionValue[Label],{width labpos[[2]],0,length labpos[[1]]},Sequence@@If[Length[labpos]>3,Drop[labpos,3],{}]],labstyle],{}]
	}
]


(* ::Subsection:: *)
(*Axes3D*)


(* ::Text:: *)
(*In order x, y, z axis*)


Options[Axes3D]={Label->{"x","y","z"},LabelPosition -> {{1.04, 0, 0}, {1.1, 0, 0}, {1.05, 0, 0}},
	AutoLengthScale->1.3,ArrowShape->{.1,.03,.01},Width->.01,Scale->Automatic,
	LabelStyle->Directive[FontFamily->"Times New Roman",FontSize->14,FontSlant->"Italic",FontWeight->"Bold",Black],
	RenderStyle->Full,Arrowheads->Automatic,BaseStyle->{},Range->{{-1,1},{-1,1},{-1,1}}};


TripleOption[expr_]:=If[Head[expr]=!=List,Table[expr,{3}],expr];


Axes3D[l:{lx_,ly_,lz_},opts:OptionsPattern[]]:=Module[{labpos,lab,labelstyle,type,heads,basestyle},
{labpos,lab,labelstyle,type,heads,basestyle}={LabelPosition,Label,LabelStyle,RenderStyle,Arrowheads,BaseStyle}/.{opts}/.Options[Axes3D];
{lab,type,heads}=TripleOption/@{lab,type,heads};
If[!Length@Dimensions@labpos>1,labpos=Table[labpos,{3}]];MapAt[RotateX[\[Pi]/2],#,1]&@MapThread[Arrow3D[#1,Label->#2,LabelPosition->#3,RenderStyle->#4,Arrowheads->#5,Sequence@@FilterRules[{opts},Options[Arrow3D]],Sequence@@FilterRules[Options[Axes3D],Options[Arrow3D]]]&,{OptionValue[Range]{{{lx,0,0},{lx,0,0}},{{0,ly,0},{0,ly,0}},{{0,0,lz},{0,0,lz}}},lab,labpos,type,heads}]
]


Axes3D[l_?NumericQ,opts___Rule]:=Axes3D[{l,l,l},opts]


Axes3D[opts___Rule]:=Axes3D[1,opts]


Axes3D[g_Graphics3D,l:{lx_,ly_,lz_},opts___Rule]:=
Show[g,Graphics3D @ Axes3D[{lx,ly,lz},opts]]


Axes3D[g_Graphics3D,l_?NumericQ,opts___Rule]:=Axes3D[g,{l,l,l},opts]


Axes3D[g_Graphics3D,opts___Rule]:=Module[{length,als},
als=AutoLengthScale/.{opts}/.Options[Axes3D];
length= als Max@Abs[PlotRange/.AbsoluteOptions[g,PlotRange]];
Axes3D[g,length,opts]
]


(* ::Subsection:: *)
(*Circle3D*)


Options[Circle3D]={Label->None,LabelPosition->{0,1.2,0},LabelStyle->{},RenderStyle->Flat,Width->.01,Arrowheads->{},ArrowShape->{.1,.03,.01},Scale->Automatic,PlotPoints->{Automatic,Automatic}}


Circle3D[r_,opts___Rule]:=Circle3D[{0,0,0},r,{0,2\[Pi]},opts]


Circle3D[p:{x_,y_,z_},r_,opts___Rule]:=Circle3D[p,r,{0,2\[Pi]},opts]


Circle3D[p:{x_,y_,z_},r_,{t1_,t2_},opts:OptionsPattern[]]:=Module[{lp=OptionValue[LabelPosition],w,rs=OptionValue[RenderStyle],ppopts,ah,t1a,t2a,ppt=OptionValue[PlotPoints]},
{w} ={Scale Width}/.{opts}/.Options[Circle3D]/.Automatic->r;
ah=fixArrowheadSpecs[OptionValue[Arrowheads],OptionValue[ArrowShape],OptionValue[Scale],Sign[t2-t1]r];
ppopts=DeleteCases[FilterRules[Join[{opts},Options[Circle3D]],Options[ParametricPlot3D]],PlotPoints->_];
{t1a,t2a}={t1,t2}-{If[Length[ah]>0&&ah[[1,1]]==0&&Sign[t2-t1]ah[[1,2,1]]<0,ah[[1,2,1]]/r,0],If[Length[ah]>0&&ah[[-1,1]]==1&&Sign[t2-t1]ah[[-1,2,1]]>0,ah[[-1,2,1]]/r,0]};
Translate[{
Switch[rs,
Full,
ParametricPlot3D[{(r+w Cos[v]) Cos[u],(r+w Cos[v]) Sin[u],w Sin[v]},{u,t1a,t2a},{v,0,2\[Pi]},Mesh->False,PlotPoints->ppt[[1]],Evaluate[Sequence@@ppopts]][[1]],
Flat,GetLine@ParametricPlot3D[r {Cos[t], Sin[t], 0},{t,t1a,t2a},PlotPoints->ppt[[1]],Evaluate@ppopts]
],
RotateZ[t1+#[[1]](t2-t1)]@TranslateX[r]@RotateX[-\[Pi]/2]@RotateY[#[[2,1]]/(2r)]@Switch[rs,Full,FatHead,Flat,FlatHead][Sequence@@#[[2]],PlotPoints->ppt[[2]]]&/@ah,
If[OptionValue[Label]=!=None,RotateZ[(t1+t2)/2+lp[[1]](t2-t1)/2]@Text[OptionValue[Label],{r lp[[2]],0,r lp[[3]]},Sequence@@If[Length[lp]>3,Drop[lp,3],{}],If[Head[#]===List,Sequence@@#,#]&@OptionValue[LabelStyle]],{}]
},
p]
]


(* ::Subsection:: *)
(*End the private context*)


End[]


(* ::Section:: *)
(*Epilog*)


(* ::Text:: *)
(*This section protects exported symbols and ends the package.*)


(* ::Subsection:: *)
(*Protect exported symbols*)


(*Protect[Evaluate[$Context<>"*"]]*)


(* ::Subsection:: *)
(*End the package context*)


EndPackage[]


(* ::Section:: *)
(*old stuff*)


(* ::Subsection::Closed:: *)
(*old Axes3D*)


(* ::Text:: *)
(*In order x, y, z axis*)


(* ::Input:: *)
(*Options[Axes3D]=Join[DeleteCases[Options[Arrow3D],BaseStyle->_],{BaseStyle->Directive[FontFamily->"Times New Roman",FontSize->14,FontSlant->"Italic",FontWeight->"Bold"],ProportionalLabelShift->True,LabelShift->Table[0.15`,{3},{3}],AxesLabels->{"x","y","z"},AutoLengthScale->1.3`,HeadRotation->{\[Pi]/2,\[Pi]/2,0}}];*)


(* ::Input:: *)
(*Axes3D[l:{lx_,ly_,lz_},opts___]:=Module[{shift,showLabels,labels,propshift,arrowopts,base,hr,sty},{propshift,shift,labels,base,hr,sty}={ProportionalLabelShift,LabelShift,AxesLabels,ArrowBase,HeadRotation,BaseStyle}/. {opts}/. Options[Axes3D];If[!MatchQ[labels,{_,_,_}],labels={"","",""}];arrowopts=Sequence@@FilterRules[Flatten[{DeleteCases[Join[{opts},Options[Axes3D]],ArrowBase->_]}],Options[FlatArrow3D]];*)
(*If[Head@base=!=List,base=Table[base,{3}]];*)
(*Flatten[{sty,ColorAll[GrayLevel[0]],RotateShape[FlatArrow3D[2 lx,ArrowBase->base[[1]],arrowopts],hr[[1]],\[Pi]/2,\[Pi]/2],RotateShape[FlatArrow3D[2 ly,ArrowBase->base[[2]],arrowopts],hr[[2]],\[Pi]/2,0],RotateShape[FlatArrow3D[2 lz,ArrowBase->base[[3]],arrowopts],hr[[3]],0,0],MapThread[Function[Text[#1,#2]],{labels,DiagonalMatrix[l (1-base)]+shift If[propshift,DiagonalMatrix[l],IdentityMatrix[3]]}]}]]*)


(* ::Input:: *)
(*Axes3D[l_?NumericQ,opts___]:=Axes3D[{l,l,l},opts]*)


(* ::Input:: *)
(*Axes3D[g_Graphics3D,l:{lx_,ly_,lz_},opts___Rule]:=*)
(*Show[g,Graphics3D @ Axes3D[{lx,ly,lz},opts]]*)


(* ::Input:: *)
(*Axes3D[g_Graphics3D,l_?NumericQ,opts___Rule]:=Axes3D[g,{l,l,l},opts]*)


(* ::Input:: *)
(*Axes3D[g_Graphics3D,opts___Rule]:=Module[{length,als},*)
(*als=AutoLengthScale/.{opts}/.Options[Axes3D];*)
(*length= als Max@Abs[PlotRange/.AbsoluteOptions[g,PlotRange]];*)
(*Axes3D[g,length,opts]*)
(*]*)


(* ::Subsection:: *)
(*old Arrow3D*)


(* ::Input:: *)
(*Options[FlatArrow3D]={ArrowHeads->1,HeadSize->1/10,ProportionalHeadSize->True,ArrowBase->0,BaseStyle->{}};*)


(* ::Input:: *)
(*FlatArrow3D[length_,opts___]:=Module[*)
(*{propsize,headsize, heads,base,startz,endz,sty},*)
(*{propsize,heads,base,sty}={ProportionalHeadSize,ArrowHeads,ArrowBase,BaseStyle}/.{opts}/.Options[FlatArrow3D];*)
(*headsize =If[propsize,length,1] HeadSize/.{opts}/.Options[FlatArrow3D];*)
(*startz=length (-base-1)/2;*)
(*endz=length (-base+1)/2;*)
(*Flatten[{sty,*)
(*Line[{{0,0,startz+If[heads>1,headsize,0]},{0,0,endz-If[heads>0,headsize,0]}}],*)
(*If[heads>0,Polygon[{{0,-.2 headsize,endz-headsize},{0,0,endz},{0,.2headsize,endz-headsize}}],{}],*)
(*If[heads>1,Polygon[{{0,-.2 headsize,startz+headsize},{0,0,startz},{0,.2headsize,startz+headsize}}],{}]*)
(*}]*)
(*]*)


(* ::Input:: *)
(*Options[FatArrow3D]={ArrowThickness->.025,ArrowBase->-1,ArrowHeads->1,HeadSize->1/10,ArrowHeadWidth->3,ProportionalHeadSize->True,ArrowLabel->"",LabelPosition->{0,10,\[Pi]/2},LabelStyle->{}};*)


(* ::Input:: *)
(*FatArrow3D[p:{x_,y_,z_},opts___Rule]:=FatArrow3D[{{0,0,0},p},opts]*)


(* ::Input:: *)
(*FatArrow3D[{p1:{x1_,y1_,z1_},p2:{x2_,y2_,z2_}},opts___Rule]:=Translate[Rotate[FatArrow3D[Sqrt[Total[(p2-p1)^2]],ArrowBase->-1,opts],{{0,0,1},(p2-p1)/Sqrt[Total[(p2-p1)^2]]}],p1]*)


(* ::Input:: *)
(*FatArrow3D[length_,opts___]:=Module[{thickness,propsize, headsize, heads, base,headwidth,label,labpos,labstyle},*)
(*{thickness,propsize,heads,base,headwidth,label,labpos,labstyle}={ArrowThickness,ProportionalHeadSize,ArrowHeads,ArrowBase,ArrowHeadWidth,ArrowLabel,LabelPosition,LabelStyle}/.{opts}/.Options[FatArrow3D];*)
(*headsize =If[propsize,length,1] HeadSize/.{opts}/.Options[FatArrow3D];*)
(*TranslateZ[-length base/2]@{EdgeForm[],*)
(*TranslateZ[-Mod[heads,2]headsize/2]@Cylinder[{{0,0,-(length-heads headsize)/2},{0,0,(length-heads headsize)/2}},thickness],*)
(*If[heads>0,TranslateZ[length/2-headsize]@{Disk3D[{0,0,0},headwidth thickness],Cone3D[headwidth thickness,headsize]},{}],*)
(*If[heads>1,TranslateZ[-(length/2-headsize)]@RotateX[\[Pi]]@{Disk3D[{0,0,0},headwidth thickness],Cone3D[headwidth thickness,headsize]},{}],RotateZ[labpos[[3]]]@Style[Text[label,{thickness labpos[[2]],0,length labpos[[1]]/2}],labstyle]*)
(*}*)
(*]*)


(* ::Subsection::Closed:: *)
(*Arc3D*)


(* ::Input:: *)
(*TakeDrop[list_,{tn_,dn_}]:=Append[Take[#,tn]&/@Most[#],Last[#]]&@Partition[list,tn+dn,tn+dn,{1,1},{}]*)


(* ::Input:: *)
(*Options[Arc3D]=Join[Options[FlatArrow3D],{PlotPoints->30,Dashing->False}];*)


(* ::Input:: *)
(*Arc3D[radius_,arclength_,opts___]:=Module[*)
(*{propsize,heads, headsize,start\[Alpha],end\[Alpha],headangle,dashing},*)
(*headangle=HeadAngle /.{opts}/.Options[Arc3D];*)
(*propsize=ProportionalHeadSize/.{opts}/.Options[Arc3D];*)
(*headsize =If[propsize,radius,1] HeadSize/.{opts}/.Options[Arc3D];*)
(*heads =Heads/.{opts}/.Options[Arc3D];*)
(*points = PlotPoints/.{opts}/.Options[Arc3D];*)
(*start\[Alpha]=-If[heads>2,headsize/radius,0];*)
(*end\[Alpha]=-arclength+If[heads>0&&arclength>0||heads>2,headsize/radius,0];*)
(*dashing=Dashing/.{opts}/.Options[Arc3D];*)
(*If[!MatchQ[dashing,{_?IntegerQ,_?IntegerQ}],dashing={points,0}];*)
(*Flatten[{*)
(*Line /@ TakeDrop[N @ Table[{radius Sin[\[Alpha]],radius Cos[\[Alpha]],0},{\[Alpha],start\[Alpha],end\[Alpha],(end\[Alpha]-start\[Alpha])/(points-1)}],dashing],*)
(*If[heads>0,*)
(*RotateShape[TranslateShape[RotateShape[Polygon[{{0,-.2 headsize,0},{-Sign[arclength]headsize,0,0},{0,.2headsize,0}}],0,headangle,0],{0,radius,0}],-arclength+Sign[arclength]headsize/radius,0,0],{}],*)
(*If[heads>1,*)
(*RotateShape[TranslateShape[RotateShape[Polygon[{{0,-.2 headsize,0},{Sign[arclength]headsize,0,0},{0,.2headsize,0}}],0,headangle,0],{0,radius,0}],-Sign[arclength]headsize/radius,0,0],*)
(*{}]*)
(*}]*)
(*]*)
(**)


(* ::Input:: *)
(*dottedarc[{thickness_,hs_,nh_},{radius_,arclength_},{x_,y_,z_},{\[Phi]_,\[Theta]_,\[Psi]_},space_]:=*)
(*(arcline=ParametricPlot3D[{radius Sin[\[Alpha]],radius Cos[\[Alpha]],0},{\[Alpha],0,-arclength},DisplayFunction->Identity][[1]];*)
(*TranslateShape[*)
(*RotateShape[*)
(*{Graphics3D[{*)
(*AbsoluteThickness[thickness],*)
(*arcline[[Table[space i-space+1,{i,Length[arcline]/space}]]]}],*)
(*If[nh>0,*)
(*RotateShape[*)
(*TranslateShape[*)
(*Graphics3D[{*)
(*EdgeForm[AbsoluteThickness[thickness]],*)
(*FaceForm[SurfaceColor[GrayLevel[0]],SurfaceColor[GrayLevel[0]]],*)
(*Polygon[{{0,-.2 hs,0},{-hs,0,0},{0,.2hs,0}}]}],*)
(*{0,radius,0}],*)
(*-arclength,0,0],*)
(*{}],*)
(*If[nh>1,*)
(*TranslateShape[*)
(*RotateShape[*)
(*Graphics3D[{*)
(*EdgeForm[AbsoluteThickness[thickness]],*)
(*FaceForm[SurfaceColor[GrayLevel[0]],SurfaceColor[GrayLevel[0]]],*)
(*Polygon[{{0,-.2 hs,0},{hs,0,0},{0,.2hs,0}}]}],*)
(*0,0,0],*)
(*{0,radius,0}],*)
(*{}]*)
(*},*)
(*\[Phi],\[Theta],\[Psi]],*)
(*{x,y,z}])*)


(* ::Subsection::Closed:: *)
(*ArcOfRevolution*)


(* ::Input:: *)
(*rotationmatrix[axis_,theta_]:=Module[{n1,n2,n3},{n1,n2,n3}=axis/Sqrt[Plus@@(axis^2)]//N;*)
(*{{n1^2+(1-n1^2) Cos[theta],n1 n2 (1-Cos[theta])+n3 Sin[theta],n1 n3 (1-Cos[theta])-n2 Sin[theta]},{n1 n2 (1-Cos[theta])-n3 Sin[theta],n2^2+(1-n2^2) Cos[theta],n2 n3 (1-Cos[theta])+n1 Sin[theta]},{n1 n3 (1-Cos[theta])+n2 Sin[theta],n2 n3 (1-Cos[theta])-n1 Sin[theta],n3^2+(1-n3^2) Cos[theta]}}//N]*)


(* ::Input:: *)
(*Options[ArcOfRevolution]={RevolutionAxis->{0,0,1},PlotPoints->15}*)


(* ::Input:: *)
(*ArcOfRevolution[point:{_?NumericQ,_?NumericQ,_?NumericQ},{tmin_?NumericQ,tmax_?NumericQ},opts___]:=Module[{axis},*)
(*{axis,points}={RevolutionAxis,PlotPoints}/.{opts}/.Options[ArcOfRevolution];*)
(*Line @ N @ Table[rotationmatrix[axis,th].point,{th,tmin,tmax,(tmax-tmin)/(points-1)}]*)
(*]*)


(* ::Subsection::Closed:: *)
(*ColorAll*)


(* ::Input:: *)
(*ColorAll[color_]:=Sequence[color,EdgeForm[color],SurfaceColor[color]]*)


(* ::Subsection:: *)
(*DoShow*)


(* ::Input:: *)
(*DoShow[g__,opts___Rule]:=Show[g,opts,DisplayFunction->$DisplayFunction]*)


(* ::Subsection::Closed:: *)
(*DontShow*)


(* ::Input:: *)
(*DontShow[g__,opts___Rule]:=Show[g,opts,DisplayFunction->Identity]*)


(* ::Subsection::Closed:: *)
(*Misc*)


(* ::Input:: *)
(*ModTextStyle[opts___]:=Union[$TextStyle/.(((#[[1]]->y_)->#&)/@{opts}),{opts}]*)


(* ::Input:: *)
(*BoldHat[x_]:=DisplayForm[StyleBox[OverscriptBox[x,AdjustmentBox["^",BoxMargins->{{0,0},{0,0}}]],Sequence@@ModTextStyle[FontWeight->"Bold"]]]*)


(* ::Input:: *)
(*vec[x_]:=DisplayForm[OverscriptBox[x,AdjustmentBox["\[RightVector]",BoxMargins->{{0,0},{0,0}}]]]*)


(* ::Input:: *)
(*vec[x_,adjust_]:=DisplayForm[OverscriptBox[x,AdjustmentBox["\[RightVector]",BoxMargins->{{0,-adjust},{0,0}}]]]*)


(* ::Input:: *)
(*hat[x_,adjust_]:=DisplayForm[OverscriptBox[x,AdjustmentBox["^",BoxMargins->{{0,-adjust},{0,0}}]]]*)


(* ::Input:: *)
(*ghost[g_]:=Graphics3D[{AbsoluteThickness[.05],Dashing[{.001,.01}],GrayLevel[0],WireFrame[g][[1]]}]*)


(* ::Input:: *)
(*ghost[g_,opts_]:=Graphics3D[Flatten[{AbsoluteThickness[.05],Dashing[{.001,.01}],GrayLevel[0],opts,WireFrame[g][[1]]}]]*)


(* ::Input:: *)
(*ChangePlotRange[plot_,range_]:=plot/.(PlotRange->_)->(PlotRange->range)*)
